package com.uas.erp.schedular.database;

import com.uas.erp.schedular.core.Constant;
import com.uas.erp.schedular.util.DateUtil;
import com.uas.erp.schedular.util.StringUtil;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.lang.reflect.Field;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Created by Pro1 on 2017/8/10.
 */
public class SqlTemplate {

    static final Map<String, Block> blocksCache = new HashMap<>();

    /**
     * 传入参数解析xml模板成可执行sql
     *
     * @param xmlStr
     * @param model
     * @return
     */
    public static String fromXml(String xmlStr, Map<String, Object> model) throws Exception {
        Document document = DocumentHelper.parseText(xmlStr);
        Block block = new Block(document.getRootElement());
        return block.toString(model);
    }

    /**
     * 传入参数解析xml模板成可执行sql
     *
     * @param xmlFile
     * @param model
     * @return
     * @throws Exception
     */
    public static String fromXmlFile(String xmlFile, Map<String, Object> model) throws Exception {
        Block block = blocksCache.get(xmlFile);
        if (null == block) {
            String template = StringUtil.copyResource(xmlFile);
            Document document = DocumentHelper.parseText(template);
            block = new Block(document.getRootElement());
            blocksCache.put(xmlFile, block);
        }
        return block.toString(model);
    }

    private static class Block {

        private String text;
        private List<Block> children;

        public Block(Element element) {
            if (element.isTextOnly()) {
                this.text = element.getText();
            } else {
                children = new ArrayList<>();
                List<Element> elements = element.elements();
                for (Element el : elements) {
                    if ("declare".equals(el.getName())) {
                        children.add(new Declare(el));
                    } else if ("iterator".equals(el.getName())) {
                        children.add(new Iterator(el));
                    } else if ("body".equals(el.getName())) {
                        children.add(new Body(el));
                    } else {
                        children.add(new Block(el));
                    }
                }
            }
        }

        /**
         * 解析形似x.y.z的属性值
         * @param prop
         * @param value
         * @return
         */
        protected Object getPropertyValue(String prop, Object value) {
            if (null == prop || null == value) {
                return value;
            }
            String key = prop;
            int index = prop.indexOf(".");
            if (index == 0) {
                prop = prop.substring(1);
                key = prop;
                index = prop.indexOf(".");
            }
            if (index > -1) {
                key = prop.substring(0, index);
                prop = prop.substring(index + 1);
            }
            try {
                Field field = value.getClass().getDeclaredField(key);
                field.setAccessible(true);
                if (index > -1) {
                    return getPropertyValue(prop, field.get(value));
                }
                return field.get(value);
            } catch (Exception e) {
                return null;
            }
        }

        private String parseValue(Object value) {
            String parsedValue = "null";
            if (null != value) {
                if (value instanceof Date) {
                    parsedValue = DateUtil.parseDateToOracleString(Constant.YMD_HMS, (Date) value);
                } else if (value instanceof Number || value instanceof Boolean) {
                    parsedValue = String.valueOf(value);
                } else if (value.toString().contains("'")) {
                    // 如果存在英文'，要进行转换处理，不然不能用sql语句存入数据库
                    parsedValue = "'" + String.valueOf(value).replaceAll("'", "''") + "'";
                } else {
                    parsedValue = "'" + value + "'";
                }
            }
            return parsedValue;
        }

        public String toString(Map<String, Object> model) {
            if (!CollectionUtils.isEmpty(children)) {
                StringBuffer buffer = new StringBuffer();
                for (Block block : children) {
                    buffer.append(block.toString(model));
                }
                return buffer.toString();
            } else {
                Pattern pattern = Pattern.compile("\\$\\{(" + StringUtils.collectionToDelimitedString(model.keySet(), "|") + ")(\\.[^}]+)?\\}");
                Matcher matcher = pattern.matcher(text);
                StringBuffer buffer = new StringBuffer();
                while (matcher.find()) {
                    String key = matcher.group(1);
                    String prop = matcher.group(2);
                    Object value = model.get(key);
                    matcher.appendReplacement(buffer, parseValue(null == prop ? value : getPropertyValue(prop, value)));
                }
                matcher.appendTail(buffer);
                return buffer.toString();
            }
        }
    }

    private static class Declare extends Block {

        public Declare(Element element) {
            super(element);
        }

        @Override
        public String toString(Map<String, Object> model) {
            return "declare " + super.toString(model);
        }
    }

    private static class Body extends Block {

        public Body(Element element) {
            super(element);
        }

        @Override
        public String toString(Map<String, Object> model) {
            return "begin " + super.toString(model) + "end;";
        }
    }

    private static class Iterator extends Block {

        private String loop;
        private String var;

        public Iterator(Element element) {
            super(element);
            this.loop = element.attributeValue("loop");
            this.var = element.attributeValue("var");
        }

        @Override
        public String toString(Map<String, Object> model) {
            StringBuffer buffer = new StringBuffer();
            Object list = null;
            int index = loop.indexOf(".");
            if (index > -1) {
                String key = loop.substring(0, index);
                String prop = loop.substring(index + 1);
                list = getPropertyValue(prop, model.get(key));
            } else {
                list = model.get(loop);
            }
            if (null != list) {
                // 循环
                Iterable<?> iterable = (Iterable<?>)list;
                java.util.Iterator<?> iterator = iterable.iterator();
                while(iterator.hasNext()) {
                    Object obj = iterator.next();
                    // 不要影响上层model
                    Map<String, Object> childModel = new HashMap<>(model);
                    childModel.put(var, obj);
                    buffer.append(super.toString(childModel));
                }
            }
            return buffer.toString();
        }

    }

}
