package com.uas.erp.schedular.publicinquiry.task;

import com.alibaba.fastjson.JSON;
import com.uas.erp.schedular.b2b.domain.Inquiry;
import com.uas.erp.schedular.b2b.domain.InquiryDecide;
import com.uas.erp.schedular.b2b.domain.InquiryDetail;
import com.uas.erp.schedular.b2b.domain.RemoteFile;
import com.uas.erp.schedular.database.SqlTemplate;
import com.uas.erp.schedular.publicproduct.domain.Prod;
import com.uas.erp.schedular.publicinquiry.domain.BatchInProduct;
import com.uas.erp.schedular.publicinquiry.domain.BatchInquiry;
import com.uas.erp.schedular.publicinquiry.domain.Enterprise;
import com.uas.erp.schedular.task.support.Method;
import com.uas.erp.schedular.task.support.TaskMapping;
import com.uas.erp.schedular.util.CollectionUtil;
import com.uas.erp.schedular.util.ContextHolder;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 轮询上传数据到公共服务平台
 *
 * Created by hejq on 2018-01-17.
 */
@Component
@TaskMapping(title = "公共询价")
public class PubInquiryTask extends AbstractTask {

    private org.slf4j.Logger logger = LoggerFactory.getLogger(this.getClass());

    /**
     * 处理物料上传的工具
     */
    protected final class ProductHandler extends AbstractTask.StatusBasedHandler<Prod> {

        public ProductHandler(List<Prod> data, Enterprise enterprise) {
            super("product", "pr_id", "PR_SENDSTATUS", "/product/erp/save?enUU=" + enterprise.getEn_uu(), data);
        }
    }

    @TaskMapping(title = "上传询价单到公共服务", fixedDelay = 30000)
    public void uploadInquiry() {
        Enterprise enterprise = getEnterprise();
        if (null != enterprise) {
            List<BatchInquiry> inquiries = getInquiryUpload();
            if (!CollectionUtils.isEmpty(inquiries)) {
                ContextHolder.setDataSize(inquiries.size());
                logger.info("inquiries size: " + inquiries.size());
                logger.info("inquiries keys: " + CollectionUtil.getKeyString(inquiries));
                String url = "/inquiry/public?enuu=" + enterprise.getEn_uu() + "&address=" + enterprise.getEn_address();
                new AbstractTask.StatusBasedHandler<BatchInquiry>("batchinquiry", "bi_id", "bi_sendstatus", url, inquiries).run();
            }
        }
    }

    @TaskMapping(title = "从公共服务下载供应商的报价信息", fixedDelay = 60000, method = Method.DOWNLOAD)
    public void downloadReply() throws Exception{
        Enterprise enterprise = getEnterprise();
        if (null != enterprise) {
            List<InquiryDetail> details = getForList("/inquiry/public/quotation?enuu=" + enterprise.getEn_uu(), InquiryDetail.class);
            if (!CollectionUtils.isEmpty(details)) {
                ContextHolder.setDataSize(details.size());
                saveInquiryReply(details);
                // 回执
                onReplySuccess(details, enterprise);
                // 发消息
                createMessageOnReplySuccess(details);
                // 附件
                saveInquiryDetailAttach(details);
            }
        }
    }

    /**
     * 创建消息
     * @param details
     */
    private void createMessageOnReplySuccess(List<InquiryDetail> details) {
        String savedIdStr = CollectionUtil.getKeyString(details, new CollectionUtil.KeyIterator<InquiryDetail>() {
            @Override
            public Object getKey(InquiryDetail obj) {
                return !"已报价".equals(obj.getId_status()) ? obj.getId_id() : null;
            }
        });
        if (!savedIdStr.isEmpty()) {
            createMessage("Inquiry", savedIdStr, "save");
        }
    }

    /**
     * 保存供应商的报价信息，存到ERP
     *
     * @author hejq
     * @date 2018-01-10 16:39
     * @param details 询价明细
     * @throws Exception
     */
    private void saveInquiryReply(List<InquiryDetail> details) throws Exception {
        Map<String, Object> model = new HashMap<>();
        model.put("inquiryDetails", details);
        // 解析sql模板
        String sql = SqlTemplate.fromXmlFile("database/template/pubInquiryReply.xml", model);
        jdbcTemplate.execute(sql);
    }

    /**
     * 修改平台里面询价单的下载状态
     *
     * @author hejq
     * @date 2018-01-10 16:40
     * @param details 询价单明细
     */
    private void onReplySuccess(List<InquiryDetail> details, Enterprise enterprise) {
        String idStr = CollectionUtil.getKeyString(details);
        post("/inquiry/public/reply/back?enuu=" + enterprise.getEn_uu(), dataWrap(idStr));
    }

    /**
     * 保存来自平台的公共询价单附件信息
     *
     * @author hejq
     * @date 2018-01-10 16:41
     * @param details 询价单明细
     */
    private void saveInquiryDetailAttach(List<InquiryDetail> details) {
        List<String> sqls = new ArrayList<String>();
        String remoteUrl = "B2B://file";
        for (InquiryDetail detail : details) {
            if (!CollectionUtils.isEmpty(detail.getFiles())) {
                StringBuffer attachIds = new StringBuffer();
                for (RemoteFile file : detail.getFiles()) {
                    int id = jdbcTemplate.generate("filepath_seq");
                    sqls.add(file.toSqlString(id, remoteUrl));
                    attachIds.append(id).append(";");
                }
                sqls.add("update inquiryAutoDetail set id_attach='" + attachIds.toString() + "' where id_id=" + detail.getId_id());
            }
        }
        jdbcTemplate.batchExecute(sqls);
    }

    /**
     * ERP询价单提交后，同步到平台进行状态更新
     *
     * @author hejq
     * @date 2018-01-10 19:32
     */
    @TaskMapping(title = "询价单提交后，上传到公共服务更新状态", fixedDelay = 60000)
    public void uploadCheckInquiry() {
        Enterprise enterprise = getEnterprise();
        if (null != enterprise) {
            List<InquiryDetail> details = jdbcTemplate.queryForBeanList("select * from (select id_id,id_ptid b2b_id_id from inquiryAutoDetail where id_status = '已转报价' and id_sendstatus = '待上传' order by id_id desc) where rownum < 100", InquiryDetail.class);
            if (!CollectionUtils.isEmpty(details)) {
                ContextHolder.setDataSize(details.size());
                new StatusBasedHandler<InquiryDetail>("inquiryAutoDetail", "id_ptid", "id_sendStatus", "/inquiry/public/checking?enuu=" + enterprise.getEn_uu(), details).run();
            }
        }
    }

    /**
     * ERP针对卖家报价信息的采纳状态传回平台
     *
     * @author hejq
     * @date 2018-01-10 19:34
     */
    @TaskMapping(title = "上传买家对报价信息是否采纳的决策到公共服务", fixedDelay = 60000)
    public void uploadInquiryDecide() {
        Enterprise enterprise = getEnterprise();
        if (null != enterprise) {
            List<InquiryDecide> replies = jdbcTemplate.queryForBeanList("select id_id,id_incode in_code,id_detno,id_isagreed id_agreed,id_ptid b2b_id from InquiryAutoDetail where nvl(ID_SENDSTATUS, ' ')<>'已上传'" +
                            " and nvl(id_isagreed,9999) <> 1 and(nvl(id_status, ' ') = '已转价格库' or nvl(id_status, ' ') = '不采纳') and rownum < 100 order by in_code,id_detno",
                    InquiryDecide.class);
            if (!CollectionUtils.isEmpty(replies)) {
                ContextHolder.setDataSize(replies.size());
                new StatusBasedHandler<InquiryDecide>("InquiryAutoDetail", "id_id", "id_sendStatus", "/inquiry/public/reply/decide?enuu=" + enterprise.getEn_uu(), replies).run();
            }
        }
    }

    /**
     * ERP单据作废信息传回平台进行操作
     *
     * @author hejq
     * @date 2018-01-10 19:34
     */
    @TaskMapping(title = "上传买家对报价信息作废的决策到公共服务", fixedDelay = 60000)
    public void uploadInquiryInvalid() {
        Enterprise enterprise = getEnterprise();
        if (null != enterprise) {
            List<Inquiry> inquiries = jdbcTemplate.queryForBeanList("select * from InquiryAuto left join InquiryAutoDetail on id_inid=in_id left join vendor on id_vendcode=ve_code where in_status = '已作废' and in_sendStatus <> '作废已上传' and nvl(in_class,' ')<>'主动报价'  and coalesce(ve_uu, '0') <> '0' and nvl(ve_b2benable,0)<>0 and rownum <= 100 order by in_code",
                    Inquiry.class);
            if (!CollectionUtils.isEmpty(inquiries)) {
                ContextHolder.setDataSize(inquiries.size());
                post("/inquiry/public/invalid?enuu=" + enterprise.getEn_uu(), dataWrap(inquiries));
                String idStr = CollectionUtil.getKeyString(inquiries);
                logger.info("公共询价单作废单据，ids: " + idStr);
                jdbcTemplate.execute("update InquiryAuto set in_sendStatus='作废已上传' where in_id in (" + idStr + ")");
            }
        }
    }

    /**
     * 获取需要上传的询价单
     *
     * @return
     */
    private List<BatchInquiry> getInquiryUpload() {
        List<BatchInquiry> inquiries = jdbcTemplate.queryForBeanList("select bi_id,bi_code,bi_date,bi_kind,bi_environment,bi_purpose,bi_pricekind,bi_recorder,bi_recorddate,bi_enddate,bi_remark,bi_pricetype,bi_recorduu,bi_attach from batchinquiry where bi_sendstatus = '待上传' and bi_status = '已审核' and bi_kind = '公开询价'  and rowNum <= 200",
                BatchInquiry.class);
        if (!CollectionUtils.isEmpty(inquiries)) {
            for (BatchInquiry inquiry : inquiries) {
                // 获取询价单明细
                List<BatchInProduct> details = jdbcTemplate.queryForBeanList("select batchinprod.*,employee.em_name,auto_inquiryprod.ycgqty,auto_inquiryprod.pr_detail,auto_inquiryprod.pr_spec,auto_inquiryprod.pr_brand,auto_inquiryprod.pr_orispeccode from batchinprod left join employee on batchinprod.bip_buyercode = employee.em_code " +
                                "left join auto_inquiryprod on batchinprod.bip_prodcode = auto_inquiryprod.pr_code where batchinprod.bip_biid = ?",
                        BatchInProduct.class, inquiry.getBi_id());
                inquiry.setInProducts(details);
                // 附件
                if (StringUtils.hasText(inquiry.getIn_attach())) {
                    inquiry.setAttaches(getAttachs(inquiry.getIn_attach().split(";")));
                }
            }
        }
        logger.info("inquiries: " + inquiries.size());
        return inquiries;
    }

    /**
     * 获取企业信息
     *
     * @return
     */
    public Enterprise getEnterprise() {
        Enterprise enterprise = jdbcTemplate.queryForBean("select en_address, en_uu, en_name from enterprise where en_uu is not null", Enterprise.class);
        logger.info("企业信息： " + JSON.toJSONString(enterprise));
        return enterprise;
    }
}
