package com.uas.erp.schedular.service;

import com.alibaba.fastjson.JSON;
import com.uas.erp.schedular.entity.Master;
import com.uas.erp.schedular.entity.MasterTaskConfigEntity;
import com.uas.erp.schedular.entity.MasterTaskConfigId;
import com.uas.erp.schedular.repository.MasterTaskConfigRepository;
import com.uas.erp.schedular.support.BackupDBFileUtils;
import com.uas.erp.schedular.task.support.MasterTaskConfig;
import com.uas.erp.schedular.task.support.RefreshableScheduledTaskRegistrar;
import com.uas.erp.schedular.task.support.TaskConfig;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * Created by Pro1 on 2017/7/26.
 */
@Service
public class ScheduledTaskService {

    @Autowired
    private MasterTaskConfigRepository masterTaskConfigRepository;

    @Autowired
    private RefreshableScheduledTaskRegistrar scheduledTaskRegistrar;

    @Autowired
    private MasterService masterService;

    private Set<TaskConfig> taskConfigsCache;

    private Set<MasterTaskConfig> masterTaskConfigsCache;

    private List<Master> mastersCache;

    private Logger logger = LoggerFactory.getLogger(this.getClass());

    /**
     * 保存 + 刷新配置缓存
     *
     * @param entity
     */
    public void save(MasterTaskConfigEntity entity) {
        entity = masterTaskConfigRepository.save(entity);
        refreshConfig(entity);
    }

    public void save(List<MasterTaskConfigEntity> entities) {
        entities = masterTaskConfigRepository.save(entities);
        BackupDBFileUtils.createFile();
        for (MasterTaskConfigEntity entity : entities) {
            refreshConfig(entity);
        }
    }

    private boolean hasMasterConfigured(String masterName) {
        if (!mastersCache.isEmpty() ) {
            for (Master master : mastersCache) {
                if (master.getMa_user().equalsIgnoreCase(masterName))
                    return true;
            }
        }
        return false;
    }

    /**
     * 查找任务配置（按账套、模块）
     *
     * @param module
     * @return
     */
    public List<MasterTaskConfigEntity> getTaskConfigs(final MasterTaskConfigId configId, final String module, final String role) {
        // 指定账套在查询时发现没有配置，主动刷一次
        if (StringUtils.hasText(configId.getMasterName()) && !hasMasterConfigured(configId.getMasterName())) {
            refreshConfigs();
        }
        return masterTaskConfigRepository.findAll(new Specification<MasterTaskConfigEntity>() {
            @Override
            public Predicate toPredicate(Root<MasterTaskConfigEntity> root, CriteriaQuery<?> query, CriteriaBuilder builder) {
                List<Predicate> predicates = new ArrayList<Predicate>();
                if (null != configId) {
                    if (!StringUtils.isEmpty(configId.getMasterName())) {
                        predicates.add(builder.equal(root.get("masterName"), configId.getMasterName()));
                    }
                    if (!StringUtils.isEmpty(configId.getBeanName())) {
                        predicates.add(builder.equal(root.get("beanName"), configId.getBeanName()));
                    }
                    if (!StringUtils.isEmpty(configId.getMethodName())) {
                        predicates.add(builder.equal(root.get("methodName"), configId.getMethodName()));
                    }
                }
                if (!StringUtils.isEmpty(module)) {
                    predicates.add(builder.equal(root.get("module"), module));
                }
                if (!StringUtils.isEmpty(role)) {
                    predicates.add(builder.equal(root.get("role"), role));
                }
                return builder.and(predicates.toArray(new Predicate[]{}));
            }
        }, new Sort("masterName", "module", "beanName", "methodName"));
    }

    /**
     * 查找任务配置
     *
     * @return
     */
    public List<MasterTaskConfigEntity> getTaskConfigs() {
        return masterTaskConfigRepository.findAll();
    }

    /**
     * 刷新entity对应的配置
     *
     * @param entity
     */
    private void refreshConfig(MasterTaskConfigEntity entity) {
        for (MasterTaskConfig config : masterTaskConfigsCache) {
            if (config.getEntity().getId().equals(entity.getId())) {
                config.setEntity(entity);
            }
        }
    }

    /**
     * 按新配置刷新到数据库
     *
     * @param configs
     */
    public boolean setConfigs(Set<TaskConfig> configs) {
        this.taskConfigsCache = configs;
        this.mastersCache = masterService.getMasters();
        return refreshAllConfigs();
    }

    /**
     * 配置更新、补全
     *
     * @return
     */
    private boolean refreshAllConfigs() {
        Set<MasterTaskConfig> masterTaskConfigs = new HashSet<>();
        if (!CollectionUtils.isEmpty(taskConfigsCache) && !CollectionUtils.isEmpty(mastersCache)) {
            List<MasterTaskConfigEntity> entities = new ArrayList<>();
            for (TaskConfig taskConfig : taskConfigsCache) {
                for (Master master : mastersCache) {
                    MasterTaskConfig masterTaskConfig = new MasterTaskConfig(taskConfig, master);
                    MasterTaskConfigEntity entity = masterTaskConfig.getEntity();
                    MasterTaskConfigEntity oldOne = masterTaskConfigRepository.findByBeanNameAndMethodNameAndMasterName(entity.getBeanName(), entity.getMethodName(), entity.getMasterName());
                    if (null != oldOne) {
                        entity.setCron(oldOne.getCron());
                        entity.setFixedDelay(oldOne.getFixedDelay());
                        entity.setFixedRate(oldOne.getFixedRate());
                        entity.setEnabled(oldOne.isEnabled());
                    }
                    entities.add(entity);
                    masterTaskConfigs.add(masterTaskConfig);
                }
            }
            // 删除可能去掉了的旧任务设置
            masterTaskConfigRepository.deleteAll();
            masterTaskConfigRepository.save(entities);
        }
        this.masterTaskConfigsCache = masterTaskConfigs;
        return !this.masterTaskConfigsCache.isEmpty();
    }

    /**
     * 刷新调度任务
     */
    @Async
    public void refreshTasks() {
        scheduledTaskRegistrar.refreshTasks(masterTaskConfigsCache);
    }

    /**
     * 每5分钟刷新一次，防账套有变
     *
     * @throws Exception
     */
    @Scheduled(cron = "0 */5 * * * *")
    public synchronized void refreshConfigs() {
        List<Master> masters = masterService.getMasters();
        if (!CollectionUtils.isEmpty(masters)) {
            if (CollectionUtils.isEmpty(mastersCache) || !masters.equals(mastersCache)) {
                mastersCache = masters;
                logger.info("refresh task configs by new masters <" + JSON.toJSONString(masters) + ">");
                refreshAllConfigs();
                refreshTasks();
            }
        }
    }

}
